<?php

declare(strict_types=1);

namespace Ig\IgNewsletterContainer\DataProcessing;


use B13\Container\Domain\Factory\Exception;
use B13\Container\Domain\Factory\PageView\Frontend\ContainerFactory;
use B13\Container\Domain\Model\Container;
use Ig\IgResponsiveImages\Utility\ResponsiveImagesUtility;
use Psr\Http\Message\ServerRequestInterface;
use TYPO3\CMS\Core\Information\Typo3Version;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Frontend\ContentObject\ContentDataProcessor;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
use TYPO3\CMS\Frontend\ContentObject\DataProcessorInterface;
use TYPO3\CMS\Frontend\ContentObject\RecordsContentObject;

class ContainerProcessor implements DataProcessorInterface
{
    /**
     * @var ContainerFactory
     */
    protected $containerFactory;

    /**
     * @var ContentDataProcessor
     */
    protected $contentDataProcessor;

    public function __construct(ContainerFactory $containerFactory, ContentDataProcessor $contentDataProcessor)
    {
        $this->containerFactory = $containerFactory;
        $this->contentDataProcessor = $contentDataProcessor;
    }

    public function process(
        ContentObjectRenderer $cObj,
        array $contentObjectConfiguration,
        array $processorConfiguration,
        array $processedData
    ): array {
        if (isset($processorConfiguration['if.']) && !$cObj->checkIf($processorConfiguration['if.'])) {
            return $processedData;
        }
        if ($processorConfiguration['contentId.'] ?? false) {
            $contentId = (int)$cObj->stdWrap($processorConfiguration['contentId'], $processorConfiguration['contentId.']);
        } elseif ($processorConfiguration['contentId'] ?? false) {
            $contentId = (int)$processorConfiguration['contentId'];
        } else {
            $contentId = (int)$cObj->data['uid'];
        }

        try {
            $container = $this->containerFactory->buildContainer($contentId);
        } catch (Exception $e) {
            // do nothing
            return $processedData;
        }

        $colPos = (int)$cObj->stdWrapValue('colPos', $processorConfiguration);
        if (empty($colPos)) {
            $allColPos = $container->getChildrenColPos();
            foreach ($allColPos as $colPos) {
                $processedData = $this->processColPos(
                    $cObj,
                    $container,
                    $colPos,
                    'children_' . $colPos,
                    $processedData,
                    $processorConfiguration
                );
            }
        } else {
            $as = $cObj->stdWrapValue('as', $processorConfiguration, 'children');
            $processedData = $this->processColPos(
                $cObj,
                $container,
                $colPos,
                $as,
                $processedData,
                $processorConfiguration
            );
        }
        return $processedData;
    }

    protected function processColPos(
        ContentObjectRenderer $cObj,
        Container $container,
        int $colPos,
        string $as,
        array $processedData,
        array $processorConfiguration
    ): array {
        $children = $container->getChildrenByColPos($colPos);

        $typo3Version = GeneralUtility::makeInstance(Typo3Version::class);
        if ($typo3Version->getMajorVersion() < 12) {
	    $contentRecordRenderer = new RecordsContentObject($cObj);
	    $contentRecordRenderer->setRequest($this->getRequest());
        } else {
            $contentRecordRenderer = new RecordsContentObject();
	    $contentRecordRenderer->setRequest($this->getRequest());
	    $contentRecordRenderer->setContentObjectRenderer($cObj);
        }
        $conf = [
            'tables' => 'tt_content',
        ];
        // IG BEGIN
        $containerRecord = $container->getContainerRecord();
        $igresponsiveimages = $processorConfiguration['igresponsiveimages.'] ?? [];
        $cols = isset($igresponsiveimages['cols']) ? intval($igresponsiveimages['cols']) : null;

        $setupUtility=GeneralUtility::makeInstance(\Ig\IgNewsletter\Utility\SetupUtility::class);
        $current = $setupUtility->getCurrent();

        // cols="2" layout="{data.layout}" frameClass="{data.frame_class}" responsive="{f:if(condition: '{flexform.mobile_layout_cols} == 1', then: '0',else: '1')}"
        $setupUtility->setTotalNumberOfCols( $igresponsiveimages['cols'] );
        $debugOutput=$setupUtility->startBox('innerBox');
        $debugOutput.=$setupUtility->startCol($setupUtility->getColWidth(), $cols);

        if( isset($containerRecord['layout'])) {
            //echo('layout='.$containerRecord['layout'].'<br />');
            $debugOutput.=$setupUtility->setLayoutConfig($containerRecord['layout'], 'outerBox');
        }
        if( isset($containerRecord['frame_class'])) {
            $setupUtility->setFrameClass($containerRecord['frame_class']);
        }
        $debugOutput.=$setupUtility->startTable();


        $totalNumberOfCols=$setupUtility->getTotalNumberOfCols();
        $layout=$setupUtility->getLayout();
        if ($cols==2 && $colPos==10 && $containerRecord['tx_ig_newsletter_column_normal_1']) {
            $colWidth = $containerRecord['tx_ig_newsletter_column_normal_1'];
        } elseif ($cols==2 && $colPos==11 && $containerRecord['tx_ig_newsletter_column_normal_2']) {
            $colWidth = $containerRecord['tx_ig_newsletter_column_normal_2'];
        } else {
            $colWidth = 0;
        }
        if ($cols) {
            $setupUtility->setCurrentCol($cols);
        }
        $col = $setupUtility->getCurrentCol();
        $setupUtility->setColWidth($colWidth);
        $colspan=intval($igresponsiveimages['colspan']);
        //$content=$setupUtility->getOuterBoxTd( $colspan, $igresponsiveimages['class'] ) . $setupUtility->debug();



        $responsiveSet = false;
        $responsiveImageUtility = GeneralUtility::makeInstance(ResponsiveImagesUtility::class);
        if( $totalNumberOfCols>=1 ) {
            $responsiveImageUtility->pushCols($totalNumberOfCols . '-spalten-nl');
            //$responsiveImageUtility->pushCols($containerRecord['CType']);
            $responsiveImageUtility->setTagWidth( $setupUtility->getWidth() );
            $responsiveSet = true;
        }

        /*
        $tagWidth = null;

        if (!empty($processorConfiguration['igresponsiveimages.'])) {

            $configName = $igresponsiveimages['config'] ? : $igresponsiveimages['cols'];
            if ($colWidth==0) {
                $colWidth = $igresponsiveimages['colWidth'];
            }
            if (($configName && $configName!='1') || $colWidth!==null) {
                if ($colWidth!==null || isset($igresponsiveimages['cols'])) {
                    $responsiveImageUtility->pushGridelements($igresponsiveimages['cols'], $colWidth, [$configName]);
                } else {
                    $responsiveImageUtility->pushCols($configName);
                }
                if (isset($igresponsiveimages['tagWidth'])) {
                    $responsiveImageUtility->setTagWidth(intval($igresponsiveimages['tagWidth']));
                }
            } else {
                $tagWidth = $responsiveImageUtility->getTagWidth();
                $responsiveImageUtility->setTagWidth(intval($igresponsiveimages['tagWidth']));
            }
            $responsiveSet = true;
        }
        */
        // IG END
        foreach ($children as &$child) {
            if ($child['l18n_parent'] > 0) {
                $conf['source'] = $child['l18n_parent'];
            } else {
                $conf['source'] = $child['uid'];
            }
            if ($child['t3ver_oid'] > 0) {
                $conf['source'] = $child['t3ver_oid'];
            }
            $child['renderedContent'] = $cObj->render($contentRecordRenderer, $conf);
            /** @var ContentObjectRenderer $recordContentObjectRenderer */
            $recordContentObjectRenderer = GeneralUtility::makeInstance(ContentObjectRenderer::class);
            $recordContentObjectRenderer->start($child, 'tt_content');
            $child = $this->contentDataProcessor->process($recordContentObjectRenderer, $processorConfiguration, $child);
        }
        // IG BEGIN
        if ($responsiveSet) {
            $responsiveImageUtility->popWidth();
            /*
            if ($tagWidth !== null) {
                $responsiveImageUtility->setTagWidth($tagWidth);
            } else {
                $responsiveImageUtility->popWidth();
            }
            */
        }

        $setupUtility->setCurrent( $current );
        //$setupUtility->setCurrentCol( $col+$colspan);
        // IG END
        $processedData[$as] = $children;
        return $processedData;
    }

    protected function getRequest(): ServerRequestInterface
    {
        return $GLOBALS['TYPO3_REQUEST'];
    }
}
